
# encoding: utf-8
# api: fpm
# title: Packfile helpers
# description: Defers further customizations to a `Packfile`
# type: delegate
# category: filter
# version: 0.1
# license: Public Domain
# 
# In order to avoid dozens of -u update filters for every variation,
# this hook utilizes a `Packfile`.
#
# The Packfile is just a Makefile, which however is run mid-packaging
# in fpm/xpm context. Specifically it's executed in the staging_path/
# when all distribution files have been assembled and arranged.
#
#  → It may reside anywhere along the /tmp/pkg-xyz-staging/ path
#    with the packaging files. Either at the root, or in a subdirectory.
#
#  → Or it's just looked for in the fpm working/startup directory.
#    In this case it's still run within /tmp/staging/ as CWD.
#
#  → In either case it's stripped from the staging_dir/ - if it was
#    ever captured in any source file/dir options.
#
#  → May execute arbitrary commands, rename stuff, compress docs, etc.
#
# As additional help, the Packfile is run with a couple of environment
# variables predefined. They're called PACK_Xyz for genericity. They're
# not meant to be tied to fpm only.
#
#  → PACK_TYPE=deb
#  → PACK_NAME=mypackage
#  → PACK_VERSION=1.2.3
#  → PACK_STAGING_PATH=/tmp/pack-deb-7310582/
#
# Avoid if possible. And beware! - If you're using anything but the
# PACK_TYPE, then you're likely mashing up build and packaging steps.
# This is really just meant for automating trivial one-off fixes.



require "fpm/package"
require "fpm/util"

# Locate Packfile, execute it in
class FPM::Package::Filter_packfile < FPM::Package
  def update(opts=nil)
    
    # Prepare PACK_TYPE env vars
    ENV["PACK_TYPE"] = @output_type
    ENV["PACK_NAME"] = @name
    ENV["PACK_VERSION"] = @version
    ENV["PACK_ARCH"] = @architecture
    ENV["PACK_STAGING"] = @staging_path
    ENV["PACK_BUILD"] = @build_path
    ENV["PACK_DIRECTORIES"] = @directories.join(" ")
    
    # Packfile from current/working/startup dir
    packfiles = []
    if File.exists? "Packfile"
      packfiles << File.absolute_path("Packfile")
    end
  
    # Start within /tmp/xyz-staging-123/ root
    ::Dir.chdir(staging_path) do
      
      # external Packfile
      packfiles.each do |pf|
        safesystem("make -f #{pf}")
      end
      
      # residual Packfiles
      ::Dir.glob("./**/Packfile").each do |pf|
        if File.exists?(pf)
          # switch to relative path
          ::Dir.chdir(File.dirname(pf)) do
            safesystem("make -f #{File.basename(pf)}")
          end
          # clean up residual
          File.unlink(pf)
        end
      end
    end # chdir staging
    
  end # def update
end # class
